############################################
# Ivan Marti-Vidal (2020), Univ. Valencia. #
############################################

# Generate a synthetic EVN experiment with PolSimulate.
# Corrupt a selection of baselines and make CLEAN images and residuals.


import numpy as np
import os



##############################################
### Run PolSimulate:

polsimulate(
  vis                =  "polsimulate_output.ms", # output ms name.
  reuse              =  False,           # Re-generate it, if it already exists.
  array_configuration    =  "EVN.cfg",    # Antenna locations.
  elevation_cutoff    =  5.0,             # Flag elevations lower than this.
  feed               =  "circular",       # Polarization feeds.
  mounts             =  [],               # Mount type for each antenna (alt-az, by default)
  ConstDt0           =  [],               # Dterms for each antenna in first pol. channel (0, by default).
  ConstDt1           =  [],               # Dterms for the second pol. channel (0, by default)
  LO                 =  5000000000.0,     # Local Oscillator (i.e., frequency of observation).
  BBs                =  [0.0],            # Baseband frequency offset(s). We only simulate one spw.
  spw_width          =  128000000.0,     # Width of the spectral window.
  nchan              =  1,                # Number of channels (per spw).
  model_image        =  [],               # CASA model image (not required).
  I                  =  [1.0],       # Fluxes of point sources to simulate (just one delta).
  Qfrac              =  [0.0],       # Fractional Stokes Q of each delta.
  Ufrac              =  [0.0],       # Fractional Stokes U of each delta.
  Vfrac              =  [0.0],       # Fractional Stokes V of each delta.
  RM                 =  [0.0],       # Rotation Measure of each delta.
  spec_index         =  [0.0],      # Spectral index of each delta.
  RAoffset           =  [0.0],   # RA offset of each delta (in degrees; i.e., centered core).
  Decoffset          =  [0.0],   # Dec offset of each delta.
  spectrum_file      =  "",                 # ascii file with the spectrum of each delta (for line simulations).
  phase_center       =  "J2000 12h00m00.08s 30d00m09.0s",  
  incell             =  "",     # Not used (i.e., no CASA image is being used in the simulation).
  inbright           =  0.0,    # Not used.
  inwidth            =  "",     # Not used.
  innu0              =  "",     # Not used.
  H0                 =  -6.0,   # Starting hour angle at array center (for VLBI, GMT).
  onsource_time      =  2.0,    # Total time on source (i.e., including overheads).
  observe_time       =  12.0,   # Total observing time.
  visib_time         =  "30s",  # Integration time at the correlator (pretty large, to save disk space).
  nscan              =  50,     # Number of scans (equally spaced).
  apply_parang       =  True,   # Apply parallactic-angle correction to the data.
  export_uvf         =  True,   # Generate uvfits file as well.
  corrupt            =  True,   # Corrupt the data with thermal noise.
  seed               =  42,     # Seed of the random number generator.
  Dt_amp             =  0.0,    # Mean amplitude for Dterm spectral noise.
  Dt_noise           =  0.001,  # Std for Dterm spectral noise.
  tau0               =  0.0,    # Atmospheric opacity (the same for all antennas).
  t_sky              =  250.0,  # Sky temperature.
  t_ground           =  270.0,  # Ground temperature.
  t_receiver         =  50.0   # Receiver temperature (the same for all antennas).
)





##############################################
### Run TCLEAN on GOOD data (i.e., with no gain corruption):


vis = 'polsimulate_output.ms'

clearcal(vis=vis,addmodel=True)

os.system('rm -rf EVN_GOOD.*')

tclean(vis=vis, imagename='EVN_GOOD',
       imsize=512, cell='0.0001arcsec',
       niter=1000, cyclefactor=0.0,
       minpsffraction=0.0, interactive=False,
       mask='box[[253pix,253pix],[259pix,259pix]]')

# Make image for talk:
## Pixels to cut the "screenshots" with image magick:
CUT = [[152,790],[632,310]]
## Generate "screenshot":
imview(raster='EVN_GOOD.residual',contour='EVN_GOOD.mask',out={'file':'EVN_GOOD.png','orient':'landscape'})

# Cut the interesting part:
os.system('convert EVN_GOOD.png -crop %ix%i+%i+%i EVN_GOOD_CROP.png'%(CUT[1][0]-CUT[0][0],-CUT[1][1]+CUT[0][1],CUT[0][0],CUT[1][1]))


###################################
## CORRUPT YS-SH:

ANTS = ['EF','WB','JB','ON','NT','SH','YS']

tb.open(vis,nomodify=False)
A1 = tb.getcol('ANTENNA1')
A2 = tb.getcol('ANTENNA2')
CORR = tb.getcol('CORRECTED_DATA')
DATA = tb.getcol('DATA')

YSSH = np.logical_or(np.logical_and(A1==ANTS.index('SH'),A2==ANTS.index('YS')),np.logical_and(A2==ANTS.index('SH'),A1==ANTS.index('YS')))

CORR[:,:,YSSH] *= 1.5

tb.putcol('CORRECTED_DATA',CORR)
tb.close()





#################################
## TCLEAN corrupted data and save image:

os.system('rm -rf EVN_YSSH_AMP.*')

tclean(vis=vis, imagename='EVN_YSSH_AMP',
       imsize=512, cell='0.0001arcsec',
       niter=1000, cyclefactor=0.0,
       minpsffraction=0.0, interactive=False,
       mask='box[[253pix,253pix],[259pix,259pix]]')


imview(raster='EVN_YSSH_AMP.residual',contour='EVN_YSSH_AMP.mask',out={'file':'EVN_YSSH_AMP.png','orient':'landscape'})
os.system('convert EVN_YSSH_AMP.png -crop %ix%i+%i+%i EVN_YSSH_AMP_CROP.png'%(CUT[1][0]-CUT[0][0],-CUT[1][1]+CUT[0][1],CUT[0][0],CUT[1][1]))




# Recover GOOD data:
CORR[:] = DATA




##################################
## Corrupt JB-WS, tclean and save image:

tb.open(vis,nomodify=False)

JBWS = np.logical_and(A1==1,A2==2)
JBWS = np.logical_or(np.logical_and(A1==ANTS.index('JB'),A2==ANTS.index('WB')),np.logical_and(A2==ANTS.index('JB'),A1==ANTS.index('WB')))

CORR[:,:,JBWS] *= np.exp(1.j*5.*np.pi/180.)

tb.putcol('CORRECTED_DATA',CORR)
tb.close()


os.system('rm -rf EVN_JBWS_PHASE.*')

tclean(vis=vis, imagename='EVN_JBWS_PHASE',
       imsize=512, cell='0.0001arcsec',
       niter=1000, cyclefactor=0.0,
       minpsffraction=0.0, interactive=False,
       mask='box[[253pix,253pix],[259pix,259pix]]')


imview(raster='EVN_JBWS_PHASE.residual',contour='EVN_JBWS_PHASE.mask',out={'file':'EVN_JBWS_PHASE.png','orient':'landscape'})
os.system('convert EVN_JBWS_PHASE.png -crop %ix%i+%i+%i EVN_JBWS_PHASE_CROP.png'%(CUT[1][0]-CUT[0][0],-CUT[1][1]+CUT[0][1],CUT[0][0],CUT[1][1]))







# Recover GOOD data:
CORR[:] = DATA



#################################
## Corrupt several baselines, tclean and save image:

tb.open(vis,nomodify=False)

CORR[:,:,JBWS] *= np.exp(1.j*2.*np.pi/180.)
CORR[:,:,YSSH] *= 1.5
CORR[:,:,A1==ANTS.index('ON')] *= np.exp(1.j*10.*np.pi/180.)
CORR[:,:,A2==ANTS.index('ON')] *= np.exp(-1.j*10.*np.pi/180.)
CORR[:,:,A1==ANTS.index('EF')] *= 1.1*np.exp(1.j*10.*np.pi/180.)
CORR[:,:,A2==ANTS.index('EF')] *= 1.1*np.exp(-1.j*10.*np.pi/180.)


tb.putcol('CORRECTED_DATA',CORR)
tb.close()


os.system('rm -rf EVN_WIBBLY.*')

tclean(vis=vis, imagename='EVN_WIBBLY',
       imsize=512, cell='0.0001arcsec',
       niter=1000, cyclefactor=0.0,
       minpsffraction=0.0, interactive=False,
       mask='box[[253pix,253pix],[259pix,259pix]]')


imview(raster='EVN_WIBBLY.residual',contour='EVN_WIBBLY.mask',out={'file':'EVN_WIBBLY.png','orient':'landscape'})
os.system('convert EVN_WIBBLY.png -crop %ix%i+%i+%i EVN_WIBBLY_CROP.png'%(CUT[1][0]-CUT[0][0],-CUT[1][1]+CUT[0][1],CUT[0][0],CUT[1][1]))


## Execute checkres (to show FFT of residuals):
os.system('rm -rf EVN_WIBBLY2.*')
tclean(vis=vis, imagename='EVN_WIBBLY2',
       imsize=2048, cell='0.0001arcsec',
       niter=1000, cyclefactor=0.0,
       minpsffraction=0.0, interactive=False,
       mask='box[[1020pix,1020pix],[1028pix,1028pix]]')


checkres(residual='EVN_WIBBLY2.residual',vis=vis)






